@@ -6,15 +6,19 @@
 import re
 import sys
 from collections import defaultdict
-from datetime import datetime, timedelta, timezone
+from datetime import datetime, timezone
+from functools import partial
+from multiprocessing.pool import Pool
 
-import dateutil.parser
 import pandas as pd
 from dateutil import parser
 from libmozdata import versions
+from libmozdata.bugzilla import Bugzilla
 from sklearn.base import BaseEstimator, TransformerMixin
 
-from bugbug import bug_snapshot, repository
+from bugbug import bug_snapshot, bugzilla, repository, utils
+
+utils.setup_libmozdata()
 
 
 def field(bug, field):
@@ -24,44 +28,43 @@ def field(bug, field):
     return None
 
 
-class single_bug_feature(object):
+class SingleBugFeature(object):
     pass
 
 
-class couple_bug_feature(object):
-    pass
-
-
-class has_str(single_bug_feature):
+class HasSTR(SingleBugFeature):
     name = "Has STR"
 
     def __call__(self, bug, **kwargs):
         return field(bug, "cf_has_str")
 
 
-class has_regression_range(single_bug_feature):
+class HasRegressionRange(SingleBugFeature):
     name = "Has Regression Range"
 
     def __call__(self, bug, **kwargs):
         return field(bug, "cf_has_regression_range")
 
 
-class has_crash_signature(single_bug_feature):
+class HasCrashSignature(SingleBugFeature):
     name = "Crash signature present"
 
     def __call__(self, bug, **kwargs):
         return "cf_crash_signature" in bug and bug["cf_crash_signature"] != ""
 
 
-class keywords(single_bug_feature):
-    def __init__(self, to_ignore=set()):
+class Keywords(SingleBugFeature):
+    def __init__(self, to_ignore=set(), prefixes_to_ignore=set()):
         self.to_ignore = to_ignore
+        self.prefixes_to_ignore = prefixes_to_ignore
 
     def __call__(self, bug, **kwargs):
         keywords = []
         subkeywords = []
         for keyword in bug["keywords"]:
-            if keyword in self.to_ignore:
+            if keyword in self.to_ignore or any(
+                keyword.startswith(prefix) for prefix in self.prefixes_to_ignore
+            ):
                 continue
 
             keywords.append(keyword)
@@ -73,19 +76,19 @@ class keywords(single_bug_feature):
         return keywords + subkeywords
 
 
-class severity(single_bug_feature):
+class Severity(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return field(bug, "severity")
 
 
-class number_of_bug_dependencies(single_bug_feature):
+class NumberOfBugDependencies(SingleBugFeature):
     name = "# of bug dependencies"
 
     def __call__(self, bug, **kwargs):
         return len(bug["depends_on"])
 
 
-class is_coverity_issue(single_bug_feature):
+class IsCoverityIssue(SingleBugFeature):
     name = "Is Coverity issue"
 
     def __call__(self, bug, **kwargs):
@@ -95,21 +98,21 @@ class is_coverity_issue(single_bug_feature):
         )
 
 
-class has_url(single_bug_feature):
+class HasURL(SingleBugFeature):
     name = "Has a URL"
 
     def __call__(self, bug, **kwargs):
         return bug["url"] != ""
 
 
-class has_w3c_url(single_bug_feature):
+class HasW3CURL(SingleBugFeature):
     name = "Has a w3c URL"
 
     def __call__(self, bug, **kwargs):
         return "w3c" in bug["url"]
 
 
-class has_github_url(single_bug_feature):
+class HasGithubURL(SingleBugFeature):
     name = "Has a GitHub URL"
 
     def __call__(self, bug, **kwargs):
@@ -138,12 +141,12 @@ def whiteboard_keywords(bug):
     return splits
 
 
-class whiteboard(single_bug_feature):
+class Whiteboard(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return whiteboard_keywords(bug)
 
 
-class patches(single_bug_feature):
+class Patches(SingleBugFeature):
     name = "# of patches"
 
     def __call__(self, bug, **kwargs):
@@ -156,24 +159,24 @@ class patches(single_bug_feature):
         )
 
 
-class landings(single_bug_feature):
+class Landings(SingleBugFeature):
     name = "# of landing comments"
 
     def __call__(self, bug, **kwargs):
         return sum(1 for c in bug["comments"] if "://hg.mozilla.org/" in c["text"])
 
 
-class product(single_bug_feature):
+class Product(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return bug["product"]
 
 
-class component(single_bug_feature):
+class Component(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return bug["component"]
 
 
-class is_mozillian(single_bug_feature):
+class IsMozillian(SingleBugFeature):
     name = "Reporter has a @mozilla email"
 
     def __call__(self, bug, **kwargs):
@@ -183,14 +186,14 @@ class is_mozillian(single_bug_feature):
         )
 
 
-class bug_reporter(single_bug_feature):
+class BugReporter(SingleBugFeature):
     name = "Bug reporter"
 
     def __call__(self, bug, **kwargs):
         return bug["creator_detail"]["email"]
 
 
-class delta_request_merge(single_bug_feature):
+class DeltaRequestMerge(SingleBugFeature):
     name = "Timespan between uplift request and following merge"
 
     def __call__(self, bug, **kwargs):
@@ -209,19 +212,53 @@ class delta_request_merge(single_bug_feature):
         return None
 
 
-class blocked_bugs_number(single_bug_feature):
+class DeltaNightlyRequestMerge(SingleBugFeature):
+    name = "Time delta between landing of the patch in Nightly and uplift request"
+
+    def __call__(self, bug, **kwargs):
+        for history in bug["history"]:
+            for change in history["changes"]:
+                if not (
+                    change["added"].startswith("approval-mozilla")
+                    and change["added"].endswith("?")
+                ):
+                    continue
+
+                uplift_request_datetime = parser.parse(history["when"])
+
+                landing_comments = Bugzilla.get_landing_comments(
+                    bug["comments"], ["nightly"]
+                )
+
+                # This will help us to find the closest landing before the uplift request
+                landing_time_list = []
+                for landing in landing_comments:
+                    landing_time = parser.parse(landing["comment"]["creation_time"])
+
+                    # Only accept if the uplift is on the future and
+                    # if the landing_time is greater than the calculated now
+                    if uplift_request_datetime >= landing_time:
+                        landing_time_list.append(landing_time)
+
+                if len(landing_time_list) > 0:
+                    time_delta = uplift_request_datetime - max(landing_time_list)
+                    return time_delta.days + time_delta.seconds / (24 * 60 * 60)
+        return None
+
+
+class BlockedBugsNumber(SingleBugFeature):
     name = "# of blocked bugs"
 
     def __call__(self, bug, **kwargs):
         return len(bug["blocks"])
 
 
-class priority(single_bug_feature):
+class Priority(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return field(bug, "priority")
 
 
-class version(single_bug_feature):
+class Version(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         if bug["version"] in ("Default", "Trunk", "trunk"):
             return "Trunk"
@@ -233,7 +270,7 @@ class version(single_bug_feature):
             return "Has Value"
 
 
-class target_milestone(single_bug_feature):
+class TargetMilestone(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         if bug["target_milestone"] == "Future":
             return "Future"
@@ -243,35 +280,35 @@ class target_milestone(single_bug_feature):
             return "Has Value"
 
 
-class has_cve_in_alias(single_bug_feature):
+class HasCVEInAlias(SingleBugFeature):
     name = "CVE in alias"
 
     def __call__(self, bug, **kwargs):
         return bug["alias"] is not None and "CVE" in bug["alias"]
 
 
-class comment_count(single_bug_feature):
+class CommentCount(SingleBugFeature):
     name = "# of comments"
 
     def __call__(self, bug, **kwargs):
         return field(bug, "comment_count")
 
 
-class comment_length(single_bug_feature):
+class CommentLength(SingleBugFeature):
     name = "Length of comments"
 
     def __call__(self, bug, **kwargs):
         return sum(len(x["text"]) for x in bug["comments"])
 
 
-class reporter_experience(single_bug_feature):
+class ReporterExperience(SingleBugFeature):
     name = "# of bugs previously opened by the reporter"
 
     def __call__(self, bug, reporter_experience, **kwargs):
         return reporter_experience
 
 
-class ever_affected(single_bug_feature):
+class EverAffected(SingleBugFeature):
     name = "status has ever been set to 'affected'"
 
     def __call__(self, bug, **kwargs):
@@ -315,7 +352,7 @@ def get_versions_statuses(bug):
     return unaffected, affected
 
 
-class affected_then_unaffected(single_bug_feature):
+class AffectedThenUnaffected(SingleBugFeature):
     name = "status has ever been set to 'affected' and 'unaffected'"
 
     def __call__(self, bug, **kwargs):
@@ -327,24 +364,24 @@ class affected_then_unaffected(single_bug_feature):
         )
 
 
-class num_words_title(single_bug_feature):
+class NumWordsTitle(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return len(bug["summary"].split())
 
 
-class num_words_comments(single_bug_feature):
+class NumWordsComments(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return sum(len(comment["text"].split()) for comment in bug["comments"])
 
 
-class has_attachment(single_bug_feature):
+class HasAttachment(SingleBugFeature):
     name = "Attachment present"
 
     def __call__(self, bug, **kwargs):
         return len(bug["attachments"]) > 0
 
 
-class has_image_attachment_at_bug_creation(single_bug_feature):
+class HasImageAttachmentAtBugCreation(SingleBugFeature):
     name = "Image attachment present at bug creation"
 
     def __call__(self, bug, **kwargs):
@@ -355,7 +392,7 @@ class has_image_attachment_at_bug_creation(single_bug_feature):
         )
 
 
-class has_image_attachment(single_bug_feature):
+class HasImageAttachment(SingleBugFeature):
     name = "Image attachment present"
 
     def __call__(self, bug, **kwargs):
@@ -364,21 +401,21 @@ class has_image_attachment(single_bug_feature):
         )
 
 
-class commit_added(single_bug_feature):
+class CommitAdded(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return sum(
             commit["added"] for commit in bug["commits"] if not commit["backedoutby"]
         )
 
 
-class commit_deleted(single_bug_feature):
+class CommitDeleted(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return sum(
             commit["deleted"] for commit in bug["commits"] if not commit["backedoutby"]
         )
 
 
-class commit_types(single_bug_feature):
+class CommitTypes(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return sum(
             (commit["types"] for commit in bug["commits"] if not commit["backedoutby"]),
@@ -386,7 +423,7 @@ class commit_types(single_bug_feature):
         )
 
 
-class commit_files_modified_num(single_bug_feature):
+class CommitFilesModifiedNum(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return sum(
             commit["files_modified_num"]
@@ -395,7 +432,7 @@ class commit_files_modified_num(single_bug_feature):
         )
 
 
-class commit_author_experience(single_bug_feature):
+class CommitAuthorExperience(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         res = [
             commit["author_experience"]
@@ -405,7 +442,7 @@ class commit_author_experience(single_bug_feature):
         return sum(res) / len(res)
 
 
-class commit_author_experience_90_days(single_bug_feature):
+class CommitAuthorExperience90Days(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         res = [
             commit["author_experience_90_days"]
@@ -415,7 +452,7 @@ class commit_author_experience_90_days(single_bug_feature):
         return sum(res) / len(res)
 
 
-class commit_reviewer_experience(single_bug_feature):
+class CommitReviewerExperience(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         res = [
             commit["reviewer_experience"]
@@ -425,7 +462,7 @@ class commit_reviewer_experience(single_bug_feature):
         return sum(res) / len(res)
 
 
-class commit_reviewer_experience_90_days(single_bug_feature):
+class CommitReviewerExperience90Days(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         res = [
             commit["reviewer_experience_90_days"]
@@ -435,12 +472,12 @@ class commit_reviewer_experience_90_days(single_bug_feature):
         return sum(res) / len(res)
 
 
-class commit_no_of_backouts(single_bug_feature):
+class CommitNoOfBackouts(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return sum(1 for commit in bug["commits"] if commit["backedoutby"])
 
 
-class components_touched(single_bug_feature):
+class ComponentsTouched(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return list(
             set(
@@ -452,7 +489,7 @@ class components_touched(single_bug_feature):
         )
 
 
-class components_touched_num(single_bug_feature):
+class ComponentsTouchedNum(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return len(
             set(
@@ -464,22 +501,27 @@ class components_touched_num(single_bug_feature):
         )
 
 
-class platform(single_bug_feature):
+class Platform(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return bug["platform"]
 
 
-class op_sys(single_bug_feature):
+class OpSys(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return bug["op_sys"]
 
 
-class is_reporter_a_developer(single_bug_feature):
+class FiledVia(SingleBugFeature):
+    def __call__(self, bug, **kwargs):
+        return bug["filed_via"]
+
+
+class IsReporterADeveloper(SingleBugFeature):
     def __call__(self, bug, author_ids, **kwargs):
-        return bug_reporter()(bug).strip() in author_ids
+        return BugReporter()(bug).strip() in author_ids
 
 
-class had_severity_enhancement(single_bug_feature):
+class HadSeverityEnhancement(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         for history in bug["history"]:
             for change in history["changes"]:
@@ -500,12 +542,11 @@ def get_time_to_fix(bug):
         return None
 
     return (
-        dateutil.parser.parse(bug["cf_last_resolved"])
-        - dateutil.parser.parse(bug["creation_time"])
+        parser.parse(bug["cf_last_resolved"]) - parser.parse(bug["creation_time"])
     ).total_seconds() / 86400
 
 
-class time_to_fix(single_bug_feature):
+class TimeToFix(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return get_time_to_fix(bug)
 
@@ -519,24 +560,40 @@ def get_time_to_assign(bug):
                 and change["added"] == "ASSIGNED"
             ):
                 return (
-                    dateutil.parser.parse(history["when"])
-                    - dateutil.parser.parse(bug["creation_time"])
+                    parser.parse(history["when"]) - parser.parse(bug["creation_time"])
                 ).total_seconds() / 86400
 
     return None
 
 
-class time_to_assign(single_bug_feature):
+class TimeToAssign(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return get_time_to_assign(bug)
 
 
-class cc_number(single_bug_feature):
+def get_time_to_close(bug):
+    """Calculate the time until closure or the time since closure for a bug."""
+    if bug["cf_last_resolved"]:
+        return (
+            parser.parse(bug["cf_last_resolved"]) - parser.parse(bug["creation_time"])
+        ).total_seconds() / 86400
+
+    return (
+        datetime.now(timezone.utc) - parser.parse(bug["creation_time"])
+    ).total_seconds() / 86400
+
+
+class TimeToClose(SingleBugFeature):
+    def __call__(self, bug, **kwargs):
+        return get_time_to_close(bug)
+
+
+class CCNumber(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return len(bug["cc"])
 
 
-class is_uplifted(single_bug_feature):
+class IsUplifted(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return any(
             change["added"].startswith("approval-mozilla")
@@ -546,109 +603,16 @@ class is_uplifted(single_bug_feature):
         )
 
 
-class resolution(single_bug_feature):
+class Resolution(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return bug["resolution"]
 
 
-class status(single_bug_feature):
+class Status(SingleBugFeature):
     def __call__(self, bug, **kwargs):
         return bug["status"]
 
 
-class couple_common_whiteboard_keywords(couple_bug_feature):
-    def __call__(self, bugs, **kwargs):
-        return [
-            keyword
-            for keyword in whiteboard_keywords(bugs[0])
-            if keyword in whiteboard_keywords(bugs[1])
-        ]
-
-
-class is_same_product(couple_bug_feature):
-    def __call__(self, bugs, **kwargs):
-        return bugs[0]["product"] == bugs[1]["product"]
-
-
-class is_same_component(couple_bug_feature):
-    def __call__(self, bugs, **kwargs):
-        return (
-            bugs[0]["product"] == bugs[1]["product"]
-            and bugs[0]["component"] == bugs[1]["component"]
-        )
-
-
-class is_same_platform(couple_bug_feature):
-    def __call__(self, bugs, **kwargs):
-        return bugs[0]["platform"] == bugs[1]["platform"]
-
-
-class is_same_version(couple_bug_feature):
-    def __call__(self, bugs, **kwargs):
-        return bugs[0]["version"] == bugs[1]["version"]
-
-
-class is_same_os(couple_bug_feature):
-    def __call__(self, bugs, **kwargs):
-        return bugs[0]["op_sys"] == bugs[1]["op_sys"]
-
-
-class is_same_target_milestone(couple_bug_feature):
-    def __call__(self, bugs, **kwargs):
-        return bugs[0]["target_milestone"] == bugs[1]["target_milestone"]
-
-
-class is_first_affected_same(couple_bug_feature):
-    def __call__(self, bugs, **kwargs):
-        version_status1 = get_versions_statuses(bugs[0])[1]
-        version_status2 = get_versions_statuses(bugs[1])[1]
-
-        if len(version_status1) > 0 and len(version_status2) > 0:
-            return min(version_status1) == min(version_status2)
-
-        return False
-
-
-class couple_delta_creation_date(couple_bug_feature):
-    def __call__(self, bugs, **kwargs):
-        delta = parser.parse(bugs[0]["creation_time"]) - parser.parse(
-            bugs[1]["creation_time"]
-        )
-        return delta / timedelta(days=1)
-
-
-class couple_common_words_summary(couple_bug_feature):
-    def __init__(self, to_ignore=set()):
-        self.to_ignore = to_ignore
-
-    def __call__(self, bugs):
-        return set(bugs[0]["summary"].split()).intersection(
-            set(bugs[1]["summary"].split())
-        )
-
-
-class couple_common_words_comments(couple_bug_feature):
-    def __init__(self, to_ignore=set()):
-        self.to_ignore = to_ignore
-
-    def __call__(self, bugs):
-        text1 = " ".join(comment["text"] for comment in bugs[0]["comments"])
-        text2 = " ".join(comment["text"] for comment in bugs[1]["comments"])
-        return set(text1.split()).intersection(set(text2.split()))
-
-
-class couple_common_keywords(couple_bug_feature):
-    def __init__(self, to_ignore=set()):
-        self.to_ignore = to_ignore
-
-    def __call__(self, bugs, **kwargs):
-        return [
-            keyword
-            for keyword in bugs[0]["keywords"]
-            if keyword in bugs[1]["keywords"] and keyword not in self.to_ignore
-        ]
-
-
 def get_author_ids():
     author_ids = set()
     for commit in repository.get_commits():
@@ -688,49 +652,20 @@ class BugExtractor(BaseEstimator, TransformerMixin):
         return self
 
     def transform(self, bugs):
-        results = []
+        bugs_iter = iter(bugs())
 
         reporter_experience_map = defaultdict(int)
         author_ids = get_author_ids() if self.commit_data else None
 
-        already_rollbacked = set()
-
         def apply_transform(bug):
-
-            is_couple = isinstance(bug, tuple)
-
-            if not is_couple:
-                bug_id = bug["id"]
-
-                if self.rollback and bug_id not in already_rollbacked:
-                    bug = bug_snapshot.rollback(bug, self.rollback_when)
-                    already_rollbacked.add(bug_id)
-
-            else:
-                bug1_id = bug[0]["id"]
-                bug2_id = bug[1]["id"]
-
-                if self.rollback:
-                    if bug1_id not in already_rollbacked:
-                        bug[0] = bug_snapshot.rollback(bug[0], self.rollback_when)
-                        already_rollbacked.add(bug1_id)
-                    if bug2_id not in already_rollbacked:
-                        bug[1] = bug_snapshot.rollback(bug[1], self.rollback_when)
-                        already_rollbacked.add(bug2_id)
-
             data = {}
 
             for feature_extractor in self.feature_extractors:
-                res = None
-                if isinstance(feature_extractor, single_bug_feature) and not is_couple:
-                    res = feature_extractor(
-                        bug,
-                        reporter_experience=reporter_experience_map[bug["creator"]],
-                        author_ids=author_ids,
-                    )
-
-                elif isinstance(feature_extractor, couple_bug_feature) and is_couple:
-                    res = feature_extractor(bug)
+                res = feature_extractor(
+                    bug,
+                    reporter_experience=reporter_experience_map[bug["creator"]],
+                    author_ids=author_ids,
+                )
 
                 if hasattr(feature_extractor, "name"):
                     feature_extractor_name = feature_extractor.name
@@ -747,57 +682,226 @@ class BugExtractor(BaseEstimator, TransformerMixin):
 
                 data[feature_extractor_name] = res
 
-            if is_couple:
-                reporter_experience_map[bug[0]["creator"]] += 1
-                reporter_experience_map[bug[1]["creator"]] += 1
+            reporter_experience_map[bug["creator"]] += 1
 
-                return {"data": data}
+            summary = bug["summary"]
+            comments = [c["text"] for c in bug["comments"]]
+            for cleanup_function in self.cleanup_functions:
+                summary = cleanup_function(summary)
+                comments = [cleanup_function(comment) for comment in comments]
 
-            else:
-                reporter_experience_map[bug["creator"]] += 1
+            return {
+                "data": data,
+                "title": summary,
+                "first_comment": "" if len(comments) == 0 else comments[0],
+                "comments": " ".join(comments),
+            }
 
-                summary = bug["summary"]
-                comments = [c["text"] for c in bug["comments"]]
-                for cleanup_function in self.cleanup_functions:
-                    summary = cleanup_function(summary)
-                    comments = [cleanup_function(comment) for comment in comments]
+        def apply_rollback(bugs_iter):
+            with Pool() as p:
+                yield from p.imap(
+                    partial(bug_snapshot.rollback, when=self.rollback_when),
+                    bugs_iter,
+                    chunksize=1024,
+                )
 
-                return {
-                    "data": data,
-                    "title": summary,
-                    "first_comment": "" if len(comments) == 0 else comments[0],
-                    "comments": " ".join(comments),
-                }
+        if self.rollback:
+            bugs_iter = apply_rollback(bugs_iter)
 
-        for bug in bugs():
-            if isinstance(bug, dict):
-                results.append(apply_transform(bug))
-            elif isinstance(bug, tuple):
-                result1 = apply_transform(bug[0])
-                result2 = apply_transform(bug[1])
+        return pd.DataFrame(apply_transform(bug) for bug in bugs_iter)
 
-                result = apply_transform(bug)
 
-                if self.merge_data:
-                    results.append(
-                        {
-                            "text": f'{result1["title"]} {result1["first_comment"]} {result2["title"]} {result2["first_comment"]}',
-                            "couple_data": result["data"],
-                        }
-                    )
-                else:
-                    results.append(
-                        {
-                            "data1": result1["data"],
-                            "data2": result2["data"],
-                            "couple_data": result["data"],
-                            "title1": result1["title"],
-                            "title2": result2["title"],
-                            "first_comment1": result1["first_comment"],
-                            "first_comment2": result2["first_comment"],
-                            "comments1": result1["comments"],
-                            "comments2": result2["comments"],
-                        }
-                    )
+class IsPerformanceBug(SingleBugFeature):
+    """Determine if the bug is related to performance based on given bug data."""
 
-        return pd.DataFrame(results)
+    name = "Is Performance Bug"
+    type_name = "performance"
+    keyword_prefixes = ("perf", "topperf", "main-thread-io")
+    whiteboard_prefixes = (
+        "[fxperf",
+        "[fxperfsize",
+        "[snappy",
+        "[pdfjs-c-performance",
+        "[pdfjs-performance",
+        "[sp3",
+    )
+
+    def __call__(
+        self,
+        bug: bugzilla.BugDict,
+        bug_map: dict[int, bugzilla.BugDict] | None = None,
+    ) -> bool:
+        if bug.get("cf_performance_impact") in ("low", "medium", "high"):
+            return True
+
+        if any(
+            keyword.startswith(prefix)
+            for keyword in bug["keywords"]
+            for prefix in self.keyword_prefixes
+        ):
+            return True
+
+        bug_whiteboard = bug["whiteboard"].lower()
+        if any(prefix in bug_whiteboard for prefix in self.whiteboard_prefixes):
+            return True
+
+        return False
+
+
+class IsMemoryBug(SingleBugFeature):
+    """Determine if the bug is related to memory based on given bug data."""
+
+    name = "Is Memory Bug"
+    type_name = "memory"
+    keyword_prefixes = ("memory-",)
+    whiteboard_prefixes = ("[overhead", "[memshrink")
+
+    def __call__(
+        self,
+        bug: bugzilla.BugDict,
+        bug_map: dict[int, bugzilla.BugDict] | None = None,
+    ) -> bool:
+        if bug_map is not None:
+            for bug_id in bug["blocks"]:
+                if bug_id not in bug_map:
+                    continue
+
+                alias = bug_map[bug_id]["alias"]
+                if alias and alias.startswith("memshrink"):
+                    return True
+
+        if any(
+            keyword.startswith(prefix)
+            for keyword in bug["keywords"]
+            for prefix in self.keyword_prefixes
+        ):
+            return True
+
+        bug_whiteboard = bug["whiteboard"].lower()
+        if any(prefix in bug_whiteboard for prefix in self.whiteboard_prefixes):
+            return True
+
+        return False
+
+
+class IsPowerBug(SingleBugFeature):
+    """Determine if the bug is related to power based on given bug data."""
+
+    name = "Is Power Bug"
+    type_name = "power"
+    keyword_prefixes = ("power",)
+    whiteboard_prefixes = ("[power",)
+
+    def __call__(
+        self,
+        bug: bugzilla.BugDict,
+        bug_map: dict[int, bugzilla.BugDict] | None = None,
+    ) -> bool:
+        if any(
+            keyword.startswith(prefix)
+            for keyword in bug["keywords"]
+            for prefix in self.keyword_prefixes
+        ):
+            return True
+
+        bug_whiteboard = bug["whiteboard"].lower()
+        if any(prefix in bug_whiteboard for prefix in self.whiteboard_prefixes):
+            return True
+
+        return False
+
+
+class IsSecurityBug(SingleBugFeature):
+    """Determine if the bug is related to security based on given bug data."""
+
+    name = "Is Security Bug"
+    type_name = "security"
+    keyword_prefixes = ("sec-", "csectype-")
+    whiteboard_prefixes = ("[client-bounty-form", "[sec-survey")
+
+    def __call__(
+        self,
+        bug: bugzilla.BugDict,
+        bug_map: dict[int, bugzilla.BugDict] | None = None,
+    ) -> bool:
+        if any(
+            keyword.startswith(prefix)
+            for keyword in bug["keywords"]
+            for prefix in self.keyword_prefixes
+        ):
+            return True
+
+        bug_whiteboard = bug["whiteboard"].lower()
+        if any(prefix in bug_whiteboard for prefix in self.whiteboard_prefixes):
+            return True
+
+        return False
+
+
+class IsCrashBug(SingleBugFeature):
+    """Determine if the bug is related to crash based on given bug data."""
+
+    name = "Is Crash Bug"
+    type_name = "crash"
+    keyword_prefixes = ("crash", "crashreportid")
+
+    def __call__(
+        self,
+        bug: bugzilla.BugDict,
+        bug_map: dict[int, bugzilla.BugDict] | None = None,
+    ) -> bool:
+        # Checking for `[@` will exclude some bugs that do not have valid
+        # signatures: https://mzl.la/46XAqRF
+        if bug.get("cf_crash_signature") and "[@" in bug["cf_crash_signature"]:
+            return True
+
+        if any(
+            keyword.startswith(prefix)
+            for keyword in bug["keywords"]
+            for prefix in self.keyword_prefixes
+        ):
+            return True
+
+        return False
+
+
+class BugTypes(SingleBugFeature):
+    """Determine bug type."""
+
+    name = "Infer Bug Type"
+    bug_type_extractors: list = [
+        IsCrashBug(),
+        IsMemoryBug(),
+        IsPerformanceBug(),
+        IsPowerBug(),
+        IsSecurityBug(),
+    ]
+
+    def __call__(
+        self,
+        bug: bugzilla.BugDict,
+        bug_map: dict[int, bugzilla.BugDict] | None = None,
+    ) -> list[str]:
+        """Infer bug types based on various bug characteristics.
+
+        Args:
+        - bug (bugzilla.BugDict): A dictionary containing bug data.
+        - bug_map (Optional[dict[int, bugzilla.BugDict]]): A mapping
+            of bug IDs to bug dictionaries. Default is None.
+
+        Returns:
+        - list[str]: A list of inferred bug types (e.g., "memory", "power",
+            "performance", "security", "crash").
+        """
+        return [
+            is_type.type_name
+            for is_type in self.bug_type_extractors
+            if is_type(bug, bug_map)
+        ]
+
+
+class BugType(SingleBugFeature):
+    """Extracts the type of the bug."""
+
+    def __call__(self, bug, **kwargs):
+        return bug["type"]
