@@ -3,11 +3,16 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
+from logging import INFO, basicConfig, getLogger
+
 import dateutil.parser
 from dateutil.relativedelta import relativedelta
 
 from bugbug import bugzilla
 
+basicConfig(level=INFO)
+logger = getLogger(__name__)
+
 
 def bool_str(val):
     assert val in ["", "0", "1"], f"Unexpected boolean value: '{val}'"
@@ -54,7 +59,7 @@ def cf_rank(val):
 # E.g. https://bugzilla.mozilla.org/rest/bug/1162372.
 def version_to_branch(version):
     if version.startswith("Firefox "):
-        return f'{version[len("Firefox "):]} Branch'
+        return f"{version[len('Firefox ') :]} Branch"
 
     return version
 
@@ -171,11 +176,8 @@ def is_expected_inconsistent_field(field, last_product, bug_id):
                 1443466,
             ]
         )
-        or (field in ["cf_has_str", "cf_has_regression_range"] and bug_id == 1_440_338)
-        or (field == "cf_has_regression_range" and bug_id == 1542185)
-        or (
-            field == "cf_has_str" and bug_id == 1462571
-        )  # TODO: Remove when https://bugzilla.mozilla.org/show_bug.cgi?id=1550104 is fixed
+        or (field == "cf_has_str" and bug_id == 1_440_338)
+        or (field == "cf_has_regression_range")  # the field was removed
     )
 
 
@@ -339,7 +341,7 @@ def is_expected_inconsistent_change_field(field, bug_id, new_value, new_value_ex
         or (
             field == "type" and bug_id == 1257155
         )  # TODO: Remove once https://bugzilla.mozilla.org/show_bug.cgi?id=1550129 is fixed.
-        or (bug_id == 1370035 and field in ("cf_has_str", "cf_has_regression_range"))
+        or (bug_id == 1370035 and field == "cf_has_str")
         or (bug_id == 1400540 and field in ("target_milestone", "status", "resolution"))
         or (bug_id == 1402929 and field == "priority")
         or (field == "whiteboard" and new_value.rstrip() == new_value_exp.rstrip())
@@ -535,11 +537,11 @@ def is_expected_inconsistent_change_flag(flag, obj_id):
 
 def rollback(bug, when=None, do_assert=False):
     def assert_or_log(msg):
-        msg = f'{msg}, in bug {bug["id"]}'
+        msg = f"{msg}, in bug {bug['id']}"
         if do_assert:
             assert False, msg
         else:
-            print(msg)
+            logger.error(msg)
 
     def parse_flag_change(change):
         parts = change.split("(")
@@ -653,7 +655,7 @@ def rollback(bug, when=None, do_assert=False):
                             break
 
                     if obj is None:
-                        assert_or_log(f'Attachment {change["attachment_id"]} not found')
+                        assert_or_log(f"Attachment {change['attachment_id']} not found")
                         continue
                 else:
                     obj = bug
@@ -745,7 +747,7 @@ def rollback(bug, when=None, do_assert=False):
 
                 if obj is None:
                     if change["comment_id"] != 14096735:
-                        assert_or_log(f'Comment {change["comment_id"]} not found')
+                        assert_or_log(f"Comment {change['comment_id']} not found")
                     continue
 
                 if obj["count"] != change["comment_count"]:
@@ -872,9 +874,8 @@ def get_inconsistencies(bugs):
     for bug in bugs:
         try:
             rollback(bug, do_assert=True)
-        except Exception as e:
-            print(bug["id"])
-            print(e)
+        except Exception:
+            logger.exception("Failed to rollback bug %s", bug["id"])
             inconsistencies.append(bug)
 
     return inconsistencies
@@ -891,6 +892,6 @@ if __name__ == "__main__":
 
     for bug in tqdm(bugzilla.get_bugs()):
         if args.verbose:
-            print(bug["id"])
+            logger.info(bug["id"])
 
         rollback(bug, do_assert=True)
