@@ -3,7 +3,6 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 import sys
-from collections import defaultdict
 from typing import Sequence
 
 import pandas as pd
@@ -15,28 +14,28 @@ EXPERIENCE_TIMESPAN = 90
 EXPERIENCE_TIMESPAN_TEXT = f"{EXPERIENCE_TIMESPAN}_days"
 
 
-class source_code_files_modified_num(object):
+class SourceCodeFilesModifiedNum(object):
     name = "# of modified code files"
 
     def __call__(self, commit, **kwargs):
         return commit["source_code_files_modified_num"]
 
 
-class other_files_modified_num(object):
+class OtherFilesModifiedNum(object):
     name = "# of modified non-code files"
 
     def __call__(self, commit, **kwargs):
         return commit["other_files_modified_num"]
 
 
-class test_files_modified_num(object):
+class TestFilesModifiedNum(object):
     name = "# of modified test files"
 
     def __call__(self, commit, **kwargs):
         return commit["test_files_modified_num"]
 
 
-class source_code_file_size(object):
+class SourceCodeFileSize(object):
     def __call__(self, commit, **kwargs):
         return {
             "Total code files size": commit["total_source_code_file_size"],
@@ -46,7 +45,7 @@ class source_code_file_size(object):
         }
 
 
-class other_file_size(object):
+class OtherFileSize(object):
     def __call__(self, commit, **kwargs):
         return {
             "Total non-code files size": commit["total_other_file_size"],
@@ -56,7 +55,7 @@ class other_file_size(object):
         }
 
 
-class test_file_size(object):
+class TestFileSize(object):
     def __call__(self, commit, **kwargs):
         return {
             "Total test files size": commit["total_test_file_size"],
@@ -66,56 +65,56 @@ class test_file_size(object):
         }
 
 
-class source_code_added(object):
+class SourceCodeAdded(object):
     name = "# of code lines added"
 
     def __call__(self, commit, **kwargs):
         return commit["source_code_added"]
 
 
-class other_added(object):
+class OtherAdded(object):
     name = "# of non-code lines added"
 
     def __call__(self, commit, **kwargs):
         return commit["other_added"]
 
 
-class test_added(object):
+class TestAdded(object):
     name = "# of lines added in tests"
 
     def __call__(self, commit, **kwargs):
         return commit["test_added"]
 
 
-class source_code_deleted(object):
+class SourceCodeDeleted(object):
     name = "# of code lines deleted"
 
     def __call__(self, commit, **kwargs):
         return commit["source_code_deleted"]
 
 
-class other_deleted(object):
+class OtherDeleted(object):
     name = "# of non-code lines deleted"
 
     def __call__(self, commit, **kwargs):
         return commit["other_deleted"]
 
 
-class test_deleted(object):
+class TestDeleted(object):
     name = "# of lines deleted in tests"
 
     def __call__(self, commit, **kwargs):
         return commit["test_deleted"]
 
 
-class functions_touched_num(object):
+class FunctionsTouchedNum(object):
     name = "# of functions touched"
 
     def __call__(self, commit, **kwargs):
         return sum(1 for f_group in commit["functions"].values() for f in f_group)
 
 
-class functions_touched_size(object):
+class FunctionsTouchedSize(object):
     def __call__(self, commit, **kwargs):
         function_sizes = [
             f["end"] - f["start"] + 1
@@ -133,7 +132,7 @@ class functions_touched_size(object):
         }
 
 
-class source_code_file_metrics(object):
+class SourceCodeFileMetrics(object):
     name = "metrics on source code file"
 
     def __call__(self, commit, **kwargs):
@@ -147,6 +146,26 @@ class source_code_file_metrics(object):
                 "halstead_n1_avg"
             ],
             "Average file number of operators": commit["metrics"]["halstead_N1_avg"],
+            "Average file length": commit["metrics"]["halstead_length_avg"],
+            "Average calculated estimated file length": commit["metrics"][
+                "halstead_estimated_program_length_avg"
+            ],
+            "Average file purity ratio": commit["metrics"]["halstead_purity_ratio_avg"],
+            "Average file vocabulary": commit["metrics"]["halstead_vocabulary_avg"],
+            "Average file volume": commit["metrics"]["halstead_volume_avg"],
+            "Average file estimated difficulty": commit["metrics"][
+                "halstead_difficulty_avg"
+            ],
+            "Average file estimated level of difficulty": commit["metrics"][
+                "halstead_level_avg"
+            ],
+            "Average file estimated effort": commit["metrics"]["halstead_effort_avg"],
+            "Average file estimated time": commit["metrics"]["halstead_time_avg"],
+            "Average file estimated number of delivered bugs": commit["metrics"][
+                "halstead_bugs_avg"
+            ],
+            "Average file number of functions": commit["metrics"]["functions_avg"],
+            "Average file number of closures": commit["metrics"]["closures_avg"],
             "Average file number of source loc": commit["metrics"]["sloc_avg"],
             "Average file number of instruction loc": commit["metrics"]["ploc_avg"],
             "Average file number of logical loc": commit["metrics"]["lloc_avg"],
@@ -169,6 +188,26 @@ class source_code_file_metrics(object):
                 "halstead_n1_max"
             ],
             "Maximum file number of operators": commit["metrics"]["halstead_N1_max"],
+            "Maximum file length": commit["metrics"]["halstead_length_max"],
+            "Maximum calculated estimated file length": commit["metrics"][
+                "halstead_estimated_program_length_max"
+            ],
+            "Maximum file purity ratio": commit["metrics"]["halstead_purity_ratio_max"],
+            "Maximum file vocabulary": commit["metrics"]["halstead_vocabulary_max"],
+            "Maximum file volume": commit["metrics"]["halstead_volume_max"],
+            "Maximum file estimated difficulty": commit["metrics"][
+                "halstead_difficulty_max"
+            ],
+            "Maximum file estimated level of difficulty": commit["metrics"][
+                "halstead_level_max"
+            ],
+            "Maximum file estimated effort": commit["metrics"]["halstead_effort_max"],
+            "Maximum file estimated time": commit["metrics"]["halstead_time_max"],
+            "Maximum file estimated number of delivered bugs": commit["metrics"][
+                "halstead_bugs_max"
+            ],
+            "Maximum file number of functions": commit["metrics"]["functions_max"],
+            "Maximum file number of closures": commit["metrics"]["closures_max"],
             "Maximum file number of source loc": commit["metrics"]["sloc_max"],
             "Maximum file number of instruction loc": commit["metrics"]["ploc_max"],
             "Maximum file number of logical loc": commit["metrics"]["lloc_max"],
@@ -191,6 +230,26 @@ class source_code_file_metrics(object):
                 "halstead_n1_min"
             ],
             "Minimum file number of operators": commit["metrics"]["halstead_N1_min"],
+            "Minimum file length": commit["metrics"]["halstead_length_min"],
+            "Minimum calculated estimated file length": commit["metrics"][
+                "halstead_estimated_program_length_min"
+            ],
+            "Minimum file purity ratio": commit["metrics"]["halstead_purity_ratio_min"],
+            "Minimum file vocabulary": commit["metrics"]["halstead_vocabulary_min"],
+            "Minimum file volume": commit["metrics"]["halstead_volume_min"],
+            "Minimum file estimated difficulty": commit["metrics"][
+                "halstead_difficulty_min"
+            ],
+            "Minimum file estimated level of difficulty": commit["metrics"][
+                "halstead_level_min"
+            ],
+            "Minimum file estimated effort": commit["metrics"]["halstead_effort_min"],
+            "Minimum file estimated time": commit["metrics"]["halstead_time_min"],
+            "Minimum file estimated number of delivered bugs": commit["metrics"][
+                "halstead_bugs_min"
+            ],
+            "Minimum file number of functions": commit["metrics"]["functions_min"],
+            "Minimum file number of closures": commit["metrics"]["closures_min"],
             "Minimum file number of source loc": commit["metrics"]["sloc_min"],
             "Minimum file number of instruction loc": commit["metrics"]["ploc_min"],
             "Minimum file number of logical loc": commit["metrics"]["lloc_min"],
@@ -213,6 +272,26 @@ class source_code_file_metrics(object):
                 "halstead_n1_total"
             ],
             "Total file number of operators": commit["metrics"]["halstead_N1_total"],
+            "Total file length": commit["metrics"]["halstead_length_total"],
+            "Total calculated estimated file length": commit["metrics"][
+                "halstead_estimated_program_length_total"
+            ],
+            "Total file purity ratio": commit["metrics"]["halstead_purity_ratio_total"],
+            "Total file vocabulary": commit["metrics"]["halstead_vocabulary_total"],
+            "Total file volume": commit["metrics"]["halstead_volume_total"],
+            "Total file estimated difficulty": commit["metrics"][
+                "halstead_difficulty_total"
+            ],
+            "Total file estimated level of difficulty": commit["metrics"][
+                "halstead_level_total"
+            ],
+            "Total file estimated effort": commit["metrics"]["halstead_effort_total"],
+            "Total file estimated time": commit["metrics"]["halstead_time_total"],
+            "Total file estimated number of delivered bugs": commit["metrics"][
+                "halstead_bugs_total"
+            ],
+            "Total file number of functions": commit["metrics"]["functions_total"],
+            "Total file number of closures": commit["metrics"]["closures_total"],
             "Total file number of source loc": commit["metrics"]["sloc_total"],
             "Total file number of instruction loc": commit["metrics"]["ploc_total"],
             "Total file number of logical loc": commit["metrics"]["lloc_total"],
@@ -256,7 +335,7 @@ def merge_function_metrics(objects):
     return metrics
 
 
-class source_code_function_metrics(object):
+class SourceCodeFunctionMetrics(object):
     name = "metrics on source code functions"
 
     def __call__(self, commit, **kwargs):
@@ -274,6 +353,28 @@ class source_code_function_metrics(object):
                 "halstead_n1_avg"
             ],
             "Average function number of operators": merged_metrics["halstead_N1_avg"],
+            "Average function length": merged_metrics["halstead_length_avg"],
+            "Average calculated estimated function length": merged_metrics[
+                "halstead_estimated_program_length_avg"
+            ],
+            "Average function purity ratio": merged_metrics[
+                "halstead_purity_ratio_avg"
+            ],
+            "Average function vocabulary": merged_metrics["halstead_vocabulary_avg"],
+            "Average function volume": merged_metrics["halstead_volume_avg"],
+            "Average function estimated difficulty": merged_metrics[
+                "halstead_difficulty_avg"
+            ],
+            "Average function estimated level of difficulty": merged_metrics[
+                "halstead_level_avg"
+            ],
+            "Average function estimated effort": merged_metrics["halstead_effort_avg"],
+            "Average function estimated time": merged_metrics["halstead_time_avg"],
+            "Average function estimated number of delivered bugs": merged_metrics[
+                "halstead_bugs_avg"
+            ],
+            "Average function number of functions": merged_metrics["functions_avg"],
+            "Average function number of closures": merged_metrics["closures_avg"],
             "Average function number of source loc": merged_metrics["sloc_avg"],
             "Average function number of instruction loc": merged_metrics["ploc_avg"],
             "Average function number of logical loc": merged_metrics["lloc_avg"],
@@ -298,6 +399,28 @@ class source_code_function_metrics(object):
                 "halstead_n1_max"
             ],
             "Maximum function number of operators": merged_metrics["halstead_N1_max"],
+            "Maximum function length": merged_metrics["halstead_length_max"],
+            "Maximum calculated estimated function length": merged_metrics[
+                "halstead_estimated_program_length_max"
+            ],
+            "Maximum function purity ratio": merged_metrics[
+                "halstead_purity_ratio_max"
+            ],
+            "Maximum function vocabulary": merged_metrics["halstead_vocabulary_max"],
+            "Maximum function volume": merged_metrics["halstead_volume_max"],
+            "Maximum function estimated difficulty": merged_metrics[
+                "halstead_difficulty_max"
+            ],
+            "Maximum function estimated level of difficulty": merged_metrics[
+                "halstead_level_max"
+            ],
+            "Maximum function estimated effort": merged_metrics["halstead_effort_max"],
+            "Maximum function estimated time": merged_metrics["halstead_time_max"],
+            "Maximum function estimated number of delivered bugs": merged_metrics[
+                "halstead_bugs_max"
+            ],
+            "Maximum function number of functions": merged_metrics["functions_max"],
+            "Maximum function number of closures": merged_metrics["closures_max"],
             "Maximum function number of source loc": merged_metrics["sloc_max"],
             "Maximum function number of instruction loc": merged_metrics["ploc_max"],
             "Maximum function number of logical loc": merged_metrics["lloc_max"],
@@ -322,6 +445,28 @@ class source_code_function_metrics(object):
                 "halstead_n1_min"
             ],
             "Minimum function number of operators": merged_metrics["halstead_N1_min"],
+            "Minimum function length": merged_metrics["halstead_length_min"],
+            "Minimum calculated estimated function length": merged_metrics[
+                "halstead_estimated_program_length_min"
+            ],
+            "Minimum function purity ratio": merged_metrics[
+                "halstead_purity_ratio_min"
+            ],
+            "Minimum function vocabulary": merged_metrics["halstead_vocabulary_min"],
+            "Minimum function volume": merged_metrics["halstead_volume_min"],
+            "Minimum function estimated difficulty": merged_metrics[
+                "halstead_difficulty_min"
+            ],
+            "Minimum function estimated level of difficulty": merged_metrics[
+                "halstead_level_min"
+            ],
+            "Minimum function estimated effort": merged_metrics["halstead_effort_min"],
+            "Minimum function estimated time": merged_metrics["halstead_time_min"],
+            "Minimum function estimated number of delivered bugs": merged_metrics[
+                "halstead_bugs_min"
+            ],
+            "Minimum function number of functions": merged_metrics["functions_min"],
+            "Minimum function number of closures": merged_metrics["closures_min"],
             "Minimum function number of source loc": merged_metrics["sloc_min"],
             "Minimum function number of instruction loc": merged_metrics["ploc_min"],
             "Minimum function number of logical loc": merged_metrics["lloc_min"],
@@ -346,6 +491,28 @@ class source_code_function_metrics(object):
                 "halstead_n1_total"
             ],
             "Total function number of operators": merged_metrics["halstead_N1_total"],
+            "Total function length": merged_metrics["halstead_length_total"],
+            "Total calculated estimated function length": merged_metrics[
+                "halstead_estimated_program_length_total"
+            ],
+            "Total function purity ratio": merged_metrics[
+                "halstead_purity_ratio_total"
+            ],
+            "Total function vocabulary": merged_metrics["halstead_vocabulary_total"],
+            "Total function volume": merged_metrics["halstead_volume_total"],
+            "Total function estimated difficulty": merged_metrics[
+                "halstead_difficulty_total"
+            ],
+            "Total function estimated level of difficulty": merged_metrics[
+                "halstead_level_total"
+            ],
+            "Total function estimated effort": merged_metrics["halstead_effort_total"],
+            "Total function estimated time": merged_metrics["halstead_time_total"],
+            "Total function estimated number of delivered bugs": merged_metrics[
+                "halstead_bugs_total"
+            ],
+            "Total function number of functions": merged_metrics["functions_total"],
+            "Total function number of closures": merged_metrics["closures_total"],
             "Total function number of source loc": merged_metrics["sloc_total"],
             "Total function number of instruction loc": merged_metrics["ploc_total"],
             "Total function number of logical loc": merged_metrics["lloc_total"],
@@ -364,7 +531,7 @@ class source_code_function_metrics(object):
         }
 
 
-class source_code_metrics_diff(object):
+class SourceCodeMetricsDiff(object):
     name = "diff in metrics on source code"
 
     def __call__(self, commit, **kwargs):
@@ -378,6 +545,28 @@ class source_code_metrics_diff(object):
                 "halstead_n1_total"
             ],
             "Diff in number of operators": commit["metrics_diff"]["halstead_N1_total"],
+            "Diff in length": commit["metrics_diff"]["halstead_length_total"],
+            "Diff in calculated estimated length": commit["metrics_diff"][
+                "halstead_estimated_program_length_total"
+            ],
+            "Diff in purity ratio": commit["metrics_diff"][
+                "halstead_purity_ratio_total"
+            ],
+            "Diff in vocabulary": commit["metrics_diff"]["halstead_vocabulary_total"],
+            "Diff in volume": commit["metrics_diff"]["halstead_volume_total"],
+            "Diff in estimated difficulty": commit["metrics_diff"][
+                "halstead_difficulty_total"
+            ],
+            "Diff in estimated level of difficulty": commit["metrics_diff"][
+                "halstead_level_total"
+            ],
+            "Diff in estimated effort": commit["metrics_diff"]["halstead_effort_total"],
+            "Diff in estimated time": commit["metrics_diff"]["halstead_time_total"],
+            "Diff in estimated number of delivered bugs": commit["metrics_diff"][
+                "halstead_bugs_total"
+            ],
+            "Diff in number of functions": commit["metrics_diff"]["functions_total"],
+            "Diff in number of closures": commit["metrics_diff"]["closures_total"],
             "Diff in number of source loc": commit["metrics_diff"]["sloc_total"],
             "Diff in number of instruction loc": commit["metrics_diff"]["ploc_total"],
             "Diff in number of logical loc": commit["metrics_diff"]["lloc_total"],
@@ -448,7 +637,7 @@ def get_exps(exp_type, commit):
     }
 
 
-class author_experience(object):
+class AuthorExperience(object):
     name = "Author experience"
 
     def __call__(self, commit, **kwargs):
@@ -465,7 +654,7 @@ class author_experience(object):
         }
 
 
-class reviewer_experience(object):
+class ReviewerExperience(object):
     def __call__(self, commit, **kwargs):
         exps = get_exps("reviewer", commit)
         return {
@@ -502,26 +691,26 @@ class reviewer_experience(object):
         }
 
 
-class reviewers_num(object):
+class ReviewersNum(object):
     name = "# of reviewers"
 
     def __call__(self, commit, **kwargs):
         return len(commit["reviewers"])
 
 
-class components(object):
+class Components(object):
     def __call__(self, commit, **kwargs):
         return commit["components"]
 
 
-class components_modified_num(object):
+class ComponentsModifiedNum(object):
     name = "# of components modified"
 
     def __call__(self, commit, **kwargs):
         return len(commit["components"])
 
 
-class component_touched_prev(object):
+class ComponentTouchedPrev(object):
     def __call__(self, commit, **kwargs):
         exps = get_exps("component", commit)
         return {
@@ -560,19 +749,19 @@ class component_touched_prev(object):
         }
 
 
-class directories(object):
+class Directories(object):
     def __call__(self, commit, **kwargs):
         return commit["directories"]
 
 
-class directories_modified_num(object):
+class DirectoriesModifiedNum(object):
     name = "# of directories modified"
 
     def __call__(self, commit, **kwargs):
         return len(commit["directories"])
 
 
-class directory_touched_prev(object):
+class DirectoryTouchedPrev(object):
     def __call__(self, commit, **kwargs):
         exps = get_exps("directory", commit)
         return {
@@ -617,38 +806,18 @@ class directory_touched_prev(object):
         }
 
 
-class files(object):
-    def __init__(self, min_freq=0.0014):
-        self.min_freq = min_freq
-
-    def fit(self, commits):
-        self.count = defaultdict(int)
-
-        self.total_commits = 0
-
-        for commit in commits:
-            self.total_commits += 1
-
-            for f in commit["files"]:
-                self.count[f] += 1
-
-        # We no longer need to store counts for files which have low frequency.
-        to_del = set(
-            f for f, c in self.count.items() if c / self.total_commits < self.min_freq
-        )
-
-        for f in to_del:
-            del self.count[f]
+class Files(object):
+    name = "files"
 
     def __call__(self, commit, **kwargs):
-        return [
-            f
-            for f in commit["files"]
-            if (self.count[f] / self.total_commits) > self.min_freq
-        ]
+        return commit["files"]
 
 
-class file_touched_prev(object):
+def _pass_through_tokenizer(doc):
+    return doc
+
+
+class FileTouchedPrev(object):
     def __call__(self, commit, **kwargs):
         exps = get_exps("file", commit)
         return {
@@ -687,7 +856,7 @@ class file_touched_prev(object):
         }
 
 
-class types(object):
+class Types(object):
     name = "file types"
 
     def __call__(self, commit, **kwargs):
@@ -818,6 +987,7 @@ class CommitExtractor(BaseEstimator, TransformerMixin):
 
         for commit in commits():
             data = {}
+            result = {"data": data}
 
             for feature_extractor in self.feature_extractors:
                 if "bug_features" in feature_extractor.__module__:
@@ -838,6 +1008,13 @@ class CommitExtractor(BaseEstimator, TransformerMixin):
                 else:
                     feature_extractor_name = feature_extractor.__class__.__name__
 
+                # FIXME: This is a workaround to pass the value to the
+                # union transformer independently. This will be dropped when we
+                # resolve https://github.com/mozilla/bugbug/issues/3876
+                if isinstance(feature_extractor, Files):
+                    result[sys.intern(feature_extractor_name)] = res
+                    continue
+
                 if isinstance(res, dict):
                     for key, value in res.items():
                         data[sys.intern(key)] = value
@@ -850,7 +1027,6 @@ class CommitExtractor(BaseEstimator, TransformerMixin):
 
                 data[sys.intern(feature_extractor_name)] = res
 
-            result = {"data": data}
             if "desc" in commit:
                 for cleanup_function in self.cleanup_functions:
                     result["desc"] = cleanup_function(commit["desc"])
