@@ -6,6 +6,7 @@
 import gzip
 import io
 import logging
+import mmap
 import os
 import pickle
 from contextlib import contextmanager
@@ -22,6 +23,10 @@ DATABASES = {}
 logger = logging.getLogger(__name__)
 
 
+class LastModifiedNotAvailable(Exception):
+    pass
+
+
 def register(path, url, version, support_files=[]):
     DATABASES[path] = {"url": url, "version": version, "support_files": support_files}
 
@@ -41,29 +46,34 @@ def exists(path):
     return os.path.exists(path)
 
 
-def is_old_schema(path):
+def is_different_schema(path):
     url = urljoin(DATABASES[path]["url"], f"{os.path.basename(path)}.version")
-    r = requests.get(url)
+    r = utils.get_session("community-tc").get(
+        url,
+        headers={
+            "User-Agent": utils.get_user_agent(),
+        },
+    )
 
     if not r.ok:
-        logger.info(f"Version file is not yet available to download for {path}")
+        logger.info("Version file is not yet available to download for %s", path)
         return True
 
     prev_version = int(r.text)
 
-    return DATABASES[path]["version"] > prev_version
+    return DATABASES[path]["version"] != prev_version
 
 
 def download_support_file(path, file_name, extract=True):
     # If a DB with the current schema is not available yet, we can't download.
-    if is_old_schema(path):
+    if is_different_schema(path):
         return False
 
     try:
         url = urljoin(DATABASES[path]["url"], file_name)
         path = os.path.join(os.path.dirname(path), file_name)
 
-        logger.info(f"Downloading {url} to {path}")
+        logger.info("Downloading %s to %s", url, path)
         updated = utils.download_check_etag(url, path)
 
         if extract and updated and path.endswith(".zst"):
@@ -73,7 +83,7 @@ def download_support_file(path, file_name, extract=True):
         return True
     except requests.exceptions.HTTPError:
         logger.info(
-            f"{file_name} is not yet available to download for {path}", exc_info=True
+            "%s is not yet available to download for %s", file_name, path, exc_info=True
         )
         return False
 
@@ -81,14 +91,14 @@ def download_support_file(path, file_name, extract=True):
 # Download and extract databases.
 def download(path, support_files_too=False, extract=True):
     # If a DB with the current schema is not available yet, we can't download.
-    if is_old_schema(path):
+    if is_different_schema(path):
         return False
 
     zst_path = f"{path}.zst"
 
     url = DATABASES[path]["url"]
     try:
-        logger.info(f"Downloading {url} to {zst_path}")
+        logger.info("Downloading %s to %s", url, zst_path)
         updated = utils.download_check_etag(url, zst_path)
 
         if extract and updated:
@@ -102,7 +112,7 @@ def download(path, support_files_too=False, extract=True):
 
         return successful
     except requests.exceptions.HTTPError:
-        logger.info(f"{url} is not yet available to download", exc_info=True)
+        logger.info("%s is not yet available to download", url, exc_info=True)
         return False
 
 
@@ -115,18 +125,22 @@ def upload(path):
 
 
 def last_modified(path):
+    if is_different_schema(path):
+        raise LastModifiedNotAvailable()
+
     url = DATABASES[path]["url"]
     last_modified = utils.get_last_modified(url)
 
     if last_modified is None:
-        raise Exception("Last-Modified is not available")
+        raise LastModifiedNotAvailable()
 
     return last_modified
 
 
 class Store:
-    def __init__(self, fh):
+    def __init__(self, fh, use_mmap=True):
         self.fh = fh
+        self.use_mmap = use_mmap
 
 
 class JSONStore(Store):
@@ -135,8 +149,29 @@ class JSONStore(Store):
             self.fh.write(orjson.dumps(elem) + b"\n")
 
     def read(self):
-        for line in io.TextIOWrapper(self.fh, encoding="utf-8"):
-            yield orjson.loads(line)
+        if self.use_mmap:
+            if os.stat(self.fh.fileno()).st_size == 0:
+                return
+
+            with mmap.mmap(self.fh.fileno(), 0, prot=mmap.PROT_READ) as buf:
+                while line := buf.readline():
+                    yield orjson.loads(line)
+        else:
+            for line in io.TextIOWrapper(self.fh, encoding="utf-8"):
+                yield orjson.loads(line)
+
+    def size(self):
+        if self.use_mmap:
+            if os.stat(self.fh.fileno()).st_size == 0:
+                return 0
+
+            with mmap.mmap(self.fh.fileno(), 0, prot=mmap.PROT_READ) as buf:
+                count = 0
+                while buf.readline():
+                    count += 1
+                return count
+        else:
+            return sum(1 for _ in io.TextIOWrapper(self.fh, encoding="utf-8"))
 
 
 class PickleStore(Store):
@@ -151,6 +186,9 @@ class PickleStore(Store):
         except EOFError:
             pass
 
+    def size(self):
+        return sum(1 for _ in self.read())
+
 
 COMPRESSION_FORMATS = ["gz", "zstd"]
 SERIALIZATION_FORMATS = {"json": JSONStore, "pickle": PickleStore}
@@ -174,21 +212,21 @@ def _db_open(path, mode):
 
     if compression == "gz":
         with gzip.GzipFile(path, mode) as f:
-            yield store_constructor(f)
+            yield store_constructor(f, use_mmap=False)
     elif compression == "zstd":
         if "w" in mode or "a" in mode:
             cctx = zstandard.ZstdCompressor()
             with open(path, mode) as f:
                 with cctx.stream_writer(f) as writer:
-                    yield store_constructor(writer)
+                    yield store_constructor(writer, use_mmap=False)
         else:
             dctx = zstandard.ZstdDecompressor()
             with open(path, mode) as f:
                 with dctx.stream_reader(f) as reader:
-                    yield store_constructor(reader)
+                    yield store_constructor(reader, use_mmap=False)
     else:
         with open(path, mode) as f:
-            yield store_constructor(f)
+            yield store_constructor(f, use_mmap=True)
 
 
 def read(path):
@@ -198,8 +236,17 @@ def read(path):
         return ()
 
     with _db_open(path, "rb") as store:
-        for elem in store.read():
-            yield elem
+        yield from store.read()
+
+
+def size(path):
+    assert path in DATABASES
+
+    if not os.path.exists(path):
+        return 0
+
+    with _db_open(path, "rb") as store:
+        return store.size()
 
 
 def write(path, elems):
