@@ -3,7 +3,10 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
+import logging
+
 import xgboost
+from imblearn.pipeline import Pipeline as ImblearnPipeline
 from imblearn.under_sampling import RandomUnderSampler
 from sklearn.compose import ColumnTransformer
 from sklearn.feature_extraction import DictVectorizer
@@ -12,25 +15,26 @@ from sklearn.pipeline import Pipeline
 from bugbug import bug_features, bugzilla, feature_cleanup, utils
 from bugbug.model import BugModel
 
+logging.basicConfig(level=logging.INFO)
+logger = logging.getLogger(__name__)
+
 
 class StepsToReproduceModel(BugModel):
     def __init__(self, lemmatization=False):
         BugModel.__init__(self, lemmatization)
 
-        self.sampler = RandomUnderSampler(random_state=0)
-
         feature_extractors = [
-            bug_features.has_regression_range(),
-            bug_features.severity(),
-            bug_features.keywords({"stepswanted"}),
-            bug_features.is_coverity_issue(),
-            bug_features.has_crash_signature(),
-            bug_features.has_url(),
-            bug_features.has_w3c_url(),
-            bug_features.has_github_url(),
-            bug_features.whiteboard(),
-            bug_features.patches(),
-            bug_features.landings(),
+            bug_features.HasRegressionRange(),
+            bug_features.Severity(),
+            bug_features.Keywords({"stepswanted"}),
+            bug_features.IsCoverityIssue(),
+            bug_features.HasCrashSignature(),
+            bug_features.HasURL(),
+            bug_features.HasW3CURL(),
+            bug_features.HasGithubURL(),
+            bug_features.Whiteboard(),
+            bug_features.Patches(),
+            bug_features.Landings(),
         ]
 
         cleanup_functions = [
@@ -45,6 +49,11 @@ class StepsToReproduceModel(BugModel):
                     "bug_extractor",
                     bug_features.BugExtractor(feature_extractors, cleanup_functions),
                 ),
+            ]
+        )
+
+        self.clf = ImblearnPipeline(
+            [
                 (
                     "union",
                     ColumnTransformer(
@@ -55,16 +64,20 @@ class StepsToReproduceModel(BugModel):
                         ]
                     ),
                 ),
+                ("sampler", RandomUnderSampler(random_state=0)),
+                (
+                    "estimator",
+                    xgboost.XGBClassifier(n_jobs=utils.get_physical_cpu_count()),
+                ),
             ]
         )
 
-        self.clf = xgboost.XGBClassifier(n_jobs=utils.get_physical_cpu_count())
-        self.clf.set_params(predictor="cpu_predictor")
-
     def get_labels(self):
         classes = {}
 
         for bug_data in bugzilla.get_bugs():
+            if bug_data["type"] != "defect":
+                continue
             if "cf_has_str" in bug_data:
                 if bug_data["cf_has_str"] == "no":
                     classes[int(bug_data["id"])] = 0
@@ -78,15 +91,13 @@ class StepsToReproduceModel(BugModel):
                         if change["removed"].startswith("stepswanted"):
                             classes[int(bug_data["id"])] = 1
 
-        print(
-            "{} bugs have no steps to reproduce".format(
-                sum(1 for label in classes.values() if label == 0)
-            )
+        logger.info(
+            "%d bugs have no steps to reproduce",
+            sum(label == 0 for label in classes.values()),
         )
-        print(
-            "{} bugs have steps to reproduce".format(
-                sum(1 for label in classes.values() if label == 1)
-            )
+        logger.info(
+            "%d bugs have steps to reproduce",
+            sum(label == 1 for label in classes.values()),
         )
 
         return classes, [0, 1]
@@ -103,4 +114,4 @@ class StepsToReproduceModel(BugModel):
         return classes
 
     def get_feature_names(self):
-        return self.extraction_pipeline.named_steps["union"].get_feature_names()
+        return self.clf.named_steps["union"].get_feature_names_out()
