@@ -6,6 +6,7 @@
 import argparse
 import concurrent.futures
 import copy
+import io
 import itertools
 import json
 import logging
@@ -1552,6 +1553,22 @@ def pull(repo_dir: str, branch: str, revision: str) -> None:
     trigger_pull()
 
 
+def import_commits(repo_dir: str, base_rev: str, patch: bytes) -> list[bytes]:
+    """Import commits from a git format-patch style patches into a Mercurial repository."""
+    with hglib.open(repo_dir) as hg:
+        logger.info("Updating to base revision %s...", base_rev)
+        hg.update(base_rev.encode("utf-8"), clean=True)
+
+        logger.info("Applying the patch ...")
+        hg.import_(patches=io.BytesIO(patch))
+
+        revs = get_revs(hg, rev_start=f"children({base_rev})::.")
+        if not revs:
+            raise Exception("Failed to retrieve commits after applying patch")
+
+        return revs
+
+
 if __name__ == "__main__":
     parser = argparse.ArgumentParser()
     parser.add_argument("repository_dir", help="Path to the repository", action="store")
