@@ -5,7 +5,8 @@
 
 import argparse
 from logging import getLogger
-from typing import List, Tuple
+
+import requests
 
 from bugbug import db
 from bugbug.github import Github, IssueDict
@@ -33,8 +34,8 @@ class Retriever(object):
         )
 
     def replace_with_private(
-        self, original_data: List[IssueDict]
-    ) -> Tuple[List[IssueDict], set]:
+        self, original_data: list[IssueDict]
+    ) -> tuple[list[IssueDict], set]:
         """Replace title and body of automatically closed public issues.
 
         Replace them with title and body of a corresponding private issue
@@ -49,30 +50,38 @@ class Retriever(object):
                     continue
 
                 owner, repo, issue_number = extracted
-                private_issue = self.github.fetch_issue_by_number(
-                    owner, repo, issue_number
-                )
-                if private_issue:
-                    item["title"] = private_issue["title"]
-                    item["body"] = private_issue["body"]
-                    updated_ids.add(item["id"])
-                    updated_issues.append(item)
+                try:
+                    private_issue = self.github.fetch_issue_by_number(
+                        owner, repo, issue_number
+                    )
+
+                    if private_issue:
+                        item["title"] = private_issue["title"]
+                        item["body"] = private_issue["body"]
+                        updated_ids.add(item["id"])
+                        updated_issues.append(item)
+
+                except requests.HTTPError as e:
+                    if e.response.status_code == 410:
+                        logger.info(e)
+                    else:
+                        raise
 
         return updated_issues, updated_ids
 
     def retrieve_issues(self) -> None:
-
         last_modified = None
         db.download(self.github.db_path)
 
         try:
             last_modified = db.last_modified(self.github.db_path)
-        except Exception:
+        except db.LastModifiedNotAvailable:
             pass
 
         if last_modified:
             logger.info(
-                f"Retrieving issues modified or created since the last run on {last_modified.isoformat()}"
+                "Retrieving issues modified or created since the last run on %s",
+                last_modified.isoformat(),
             )
             data = self.github.fetch_issues_updated_since_timestamp(
                 last_modified.isoformat()
