@@ -8,6 +8,8 @@ import os
 import subprocess
 import tempfile
 from collections import defaultdict
+from concurrent.futures import ALL_COMPLETED, ThreadPoolExecutor, wait
+from datetime import datetime, timedelta
 from logging import INFO, basicConfig, getLogger
 
 import requests
@@ -32,6 +34,32 @@ def download_dbs():
     db.download(CI_FAILURES_DB)
 
 
+@tenacity.retry(
+    wait=tenacity.wait_exponential(multiplier=2, min=2),
+    stop=tenacity.stop_after_attempt(7),
+    reraise=True,
+)
+def query_redash(start_date, end_date):
+    r = utils.get_session("redash").post(
+        "https://sql.telemetry.mozilla.org/api/queries/111789/results",
+        json={
+            "parameters": {
+                "startdate": start_date.strftime("%Y-%m-%d"),
+                "enddate": end_date.strftime("%Y-%m-%d"),
+            },
+            "max_age": 1800,
+        },
+        headers={"Authorization": f"Key {utils.get_secret('REDASH_API_KEY')}"},
+    )
+    r.raise_for_status()
+
+    result = r.json()
+    if "query_result" not in result:
+        raise tenacity.TryAgain
+
+    return result["query_result"]["data"]["rows"]
+
+
 def get_fixed_by_commit_pushes():
     logger.info("Get previously found failures...")
     fixed_by_commit_pushes = {}
@@ -41,12 +69,17 @@ def get_fixed_by_commit_pushes():
             "commits": [],
         }
 
-    r = requests.get(
-        "https://sql.telemetry.mozilla.org/api/queries/111789/results.json?api_key={}".format(
-            utils.get_secret("REDASH_API_KEY")
-        )
-    )
-    fixed_by_commit_elements = r.json()["query_result"]["data"]["rows"]
+    fixed_by_commit_elements = []
+
+    end = today = datetime.today()
+    # Treeherder stores 120 days of data.
+    start = end - timedelta(days=120)
+
+    while start < today:
+        end = min(start + timedelta(days=7), today)
+        logger.info(f"Retrieving 'fixed by commit' data between {start} and {end}...")
+        fixed_by_commit_elements += query_redash(start, end)
+        start = end
 
     fixed_by_commit_elements = [
         element
@@ -55,6 +88,9 @@ def get_fixed_by_commit_pushes():
     ]
 
     for element in fixed_by_commit_elements:
+        if element["bug_id"] is None:
+            continue
+
         bug_id = int(element["bug_id"])
 
         if bug_id not in fixed_by_commit_pushes:
@@ -158,7 +194,7 @@ def retrieve_logs(fixed_by_commit_pushes, upload):
             if os.path.exists(log_path) or os.path.exists(log_zst_path):
                 continue
 
-            if upload and utils.exists_s3(log_path):
+            if upload and utils.exists_s3(log_zst_path):
                 continue
 
             try:
@@ -242,7 +278,7 @@ def generate_diffs(repo_url, repo_path, fixed_by_commit_pushes, upload):
         if os.path.exists(diff_path) or os.path.exists(diff_zst_path):
             continue
 
-        if upload and utils.exists_s3(diff_path):
+        if upload and utils.exists_s3(diff_zst_path):
             continue
 
         diff = diff_failure_vs_fix(
@@ -300,17 +336,34 @@ def main() -> None:
         "Retrieve CI failures, their logs and generate the diffs that fixed them"
     )
     parser = argparse.ArgumentParser(description=description)
-    parser.add_argument("repo_path", help="Path to git repository.")
     parser.add_argument("repo_url", help="Repository URL.")
+    parser.add_argument("repo_path", help="Path to git repository.")
     parser.add_argument("--upload", help="Upload logs and diffs.", action="store_true")
     args = parser.parse_args()
 
     download_dbs()
 
     fixed_by_commit_pushes = get_fixed_by_commit_pushes()
-    retrieve_logs(fixed_by_commit_pushes, args.upload)
 
-    generate_diffs(args.repo_url, args.repo_path, fixed_by_commit_pushes, args.upload)
+    with ThreadPoolExecutor(max_workers=2) as executor:
+        futures = (
+            executor.submit(retrieve_logs, fixed_by_commit_pushes, args.upload),
+            executor.submit(
+                generate_diffs,
+                args.repo_url,
+                args.repo_path,
+                fixed_by_commit_pushes,
+                args.upload,
+            ),
+        )
+
+        done, _ = wait(futures, return_when=ALL_COMPLETED)
+
+        for task in done:
+            try:
+                _ = task.result()
+            except Exception as e:
+                raise e
 
     write_results(fixed_by_commit_pushes)
 
