@@ -9,7 +9,7 @@ import logging
 import time
 import traceback
 from datetime import datetime
-from typing import Any, Dict, Optional
+from typing import Any
 
 import matplotlib.pyplot as plt
 import mozci.push
@@ -19,6 +19,7 @@ from tqdm import tqdm
 
 from bugbug import db, test_scheduling, utils
 
+logging.basicConfig(level=logging.INFO)
 logger = logging.getLogger(__name__)
 
 
@@ -32,7 +33,7 @@ db.register(
 
 def analyze_shadow_schedulers(
     push: mozci.push.Push,
-) -> Dict[str, Any]:
+) -> dict[str, Any]:
     schedulers = []
 
     for name, config_groups in push.generate_all_shadow_scheduler_config_groups():
@@ -62,7 +63,7 @@ def go(months: int) -> None:
         scheduler_stat["id"] for scheduler_stat in db.read(SHADOW_SCHEDULER_STATS_DB)
     }
     logger.info(
-        f"Already gathered statistics for {len(known_scheduler_stats)} pushes..."
+        "Already gathered statistics for %d pushes...", len(known_scheduler_stats)
     )
 
     to_date = datetime.utcnow() - relativedelta(days=3)
@@ -74,7 +75,7 @@ def go(months: int) -> None:
     )
 
     pushes = [push for push in pushes if push.rev not in known_scheduler_stats]
-    logger.info(f"{len(pushes)} left to analyze")
+    logger.info("%d left to analyze", len(pushes))
 
     def compress_and_upload() -> None:
         utils.zstd_compress(SHADOW_SCHEDULER_STATS_DB)
@@ -189,7 +190,7 @@ def plot_graphs(granularity: str) -> None:
         if scheduler_stat["id"] not in regressions_by_rev:
             continue
 
-        obj: Dict[str, Any] = {
+        obj: dict[str, Any] = {
             "date": datetime.utcfromtimestamp(scheduler_stat["date"]),
         }
 
@@ -253,7 +254,7 @@ def plot_graphs(granularity: str) -> None:
 
 
 def print_uncaught(
-    granularity: str, scheduler1: str, scheduler2: Optional[str] = None
+    granularity: str, scheduler1: str, scheduler2: str | None = None
 ) -> None:
     push_data_db = (
         test_scheduling.PUSH_DATA_GROUP_DB
@@ -298,19 +299,32 @@ def print_uncaught(
 
         if len(caught_by_scheduler[scheduler1]) == 0:
             if scheduler2 is not None and scheduler2 not in caught_by_scheduler:
-                print(
-                    f"{scheduler1} didn't catch any of the {len(regressions)} regressions on {rev}"
+                logger.info(
+                    "%s didn't catch any of the %d regressions on %s",
+                    scheduler1,
+                    len(regressions),
+                    rev,
                 )
             elif scheduler2 is not None and len(caught_by_scheduler[scheduler2]) == 0:
-                print(
-                    f"{scheduler1} and {scheduler2} didn't catch any of the {len(regressions)} regressions on {rev}"
+                logger.info(
+                    "%s and %s didn't catch any of the %d regressions on %s",
+                    scheduler1,
+                    scheduler2,
+                    len(regressions),
+                    rev,
                 )
             else:
-                print(
-                    f"{scheduler1} didn't catch any of the {len(regressions)} regressions on {rev}, while {scheduler2} did"
+                logger.info(
+                    "%s didn't catch any of the %d regressions on %s, while %s did",
+                    scheduler1,
+                    len(regressions),
+                    rev,
+                    scheduler2,
                 )
-            print(f"Regressions: {regressions}")
-            print(f"Scheduled by {scheduler1}: {scheduled_by_scheduler[scheduler1]}")
+            logger.info("Regressions: %s", regressions)
+            logger.info(
+                "Scheduled by %s: %s", scheduler1, scheduled_by_scheduler[scheduler1]
+            )
 
 
 def main() -> None:
